@@ -8,18 +8,18 @@ use Moose::Meta::Class;
 use B::Hooks::EndOfScope;
 use MooseX::Method::Signatures;
 
-our $VERSION = '0.01_01';
+our $VERSION = '0.01';
 
-our ($Declarator, $Offset, @Roles);
+our ($Declarator, $Offset, %Outer_Stack, @Roles);
 
 sub import {
-    my ($class, $type) = @_;
+    my ($class, $type, %args) = @_;
     $type ||= '';
 
     my $caller = caller();
 
-    my @blocks    = qw/class role/;
-    my @modifiers = qw/before after around override augment/;
+    my @blocks       = qw/class role/;
+    my @modifiers    = qw/before after around override augment/;
 
     my @exported = @blocks;
 
@@ -33,6 +33,11 @@ sub import {
         });
 
         push @exported, @modifiers;
+
+        if (exists $args{file}) {
+            $Outer_Stack{ $args{file} } ||= [];
+            push @{ $Outer_Stack{ $args{file} } }, $args{outer_package};
+        }
     }
 
     {
@@ -216,17 +221,17 @@ sub class_parser {
     my ($package, $anon);
 
     if (defined $name) {
+        use Data::Dump qw/dump/;
         $package = $name;
-        my $stash = Devel::Declare::get_curstash_name();
-        $package = join('::', $stash, $name)
-            unless $stash eq 'main';
+        my $outer_stack = $Outer_Stack{ (caller(1))[1] };
+        $package = join('::', $outer_stack->[-1], $package) if $outer_stack && @{ $outer_stack };
     }
     else {
         $anon = Moose::Meta::Class->create_anon_class;
         $package = $anon->name;
     }
 
-    my $inject = qq/package ${package}; use MooseX::Declare 'inner'; /;
+    my $inject = qq/package ${package}; use MooseX::Declare 'inner', outer_package => '${package}', file => __FILE__; /;
     my $inject_after = '';
 
     if ($Declarator eq 'class') {
@@ -242,6 +247,8 @@ sub class_parser {
     $inject .= 'use namespace::clean -except => [qw/meta/];';
     $inject .= options_unwrap($options);
 
+    $inject_after .= 'BEGIN { my $file = __FILE__; my $outer = $MooseX::Declare::Outer_Stack{$file}; pop @{ $outer } if $outer && @{ $outer } }';
+
     if (defined $name) {
         $inject .= scope_injector_call($inject_after);
     }
